\name{SpectralSynthesis}
\alias{SpectralSynthesis}
\title{
Generate a Landscape Using Fractional Brownian Motion
}
\description{
A function to generate a multidimensional landscape as a realisation of a fractional Brownian motion process.  This function uses an algorithm adapted from Keitt (2000) and corrected to adhere to a complex conjugacy condition described in Peitgen and Saupe (1988).
}
\usage{
SpectralSynthesis(DimExtents, Hursts, RescaleVals = c(0,1))
}
\arguments{
  \item{DimExtents}{
An integer vector with the length of the desired output in each dimension.  \code{length(DimExtents)} gives the number of dimensions in the output.
}
  \item{Hursts}{
A vector of Hurst exponents in each dimension.  If \code{length(DimExtents)} is greater than \code{length(Hursts)} then the values are recycled.
}
  \item{RescaleVals}{
The outputs are scaled so that all values fall within \code{min(RescaleVals)} and \code{max(RescaleVals)}.  If \code{RescaleVals} is \code{NULL} then the outputs are not rescaled and raw outputs are provided instead.
}
}
\value{
A vector of length \code{prod(DimExtents)} and with a dimensional extent attribute set to \code{DimExtents}.  Each element contains the (possibly rescaled) output from a realisation of fractional Brownian motion.
}
\references{
Keitt, T. H. (2000) Spectral representation of neutral landscapes.  \emph{Landscape Ecology}, 15, 479-493.

Peitgen, H. O. and Saupe, D. (1988) \emph{The Science of Fractal Images}, Springer-Verlag.

With, K. A. (1997) The application of neutral landscape models in conservation biology.  \emph{Conservation Biology}, 11, 1069-1080.

With, K. A. and King A. W. (1997) The use and misuse of neutral landscapes models in ecology.  \emph{Oikos}, 79, 219-229.
}
\author{
Joseph Chipperfield <\email{joechip90@googlemail.com}>
}
\note{
It its current implementation, this algorithm creates a landscape with a length in each dimension equal to the smallest power of two that is greater than or equal to the largest value in \code{DimExtents}.  The final landscape is created by taking the first \code{DimExtents} cells in the relevant dimension.  This means that it will take just as long to create a landscape that has a \code{DimExtents} value of \code{c(4,4,100)} than it will to create a landscape with a \code{DimExtents} of \code{c(128,128,128)}.  Because spectral synthesis methods are employed, the outputs from this function are periodic.  If this is not desirable then it may be neccessary to generate landscapes that are two or three times larger than required and simply take the first few cells in each dimension.
}
\seealso{
\code{\link{PercolationMap}}
\code{\link{HierarchicalCurdling}}
}
\examples{
# Create a 128 x 128 landscape with a Hurst exponent of 0.5 in each dimension.
# Output values are scaled between 0 and 5
dims.in <- rep(128,2)		# Dimension extent input vector
hursts <- rep(0.5,2)		# 0.5 Hurst in both spatial dimensions
rescaleVals <- c(0,5)		# Rescale the outputs between 0 and 5
output.map <- SpectralSynthesis(dims.in, hursts, rescaleVals)
image(output.map, col = heat.colors(100))
}
\keyword{models}
