#include "stdafx.h"
#include "ExtNumber.h"

//If the file is being compiled with a Visual C++ compiler then set it to be included once in the program only
//and to temporarily disable exception specification warnings generated as a result of non-standard behaviour
#ifdef _MSC_VER
	#pragma once
	#pragma warning( disable : 4290 )
#endif

namespace StdExtensions
{	// ----------------------------------------
	//  METHODS FOR THE EXTENDED BOOLEAN CLASS
	// ----------------------------------------

	// Initialisation function using a missing data value
	void ExtBool::Initialise(SpecialValues eSpecIn) throw(Exception)
	{
		Initialise();
		if(eSpecIn != e_MissingValue)
		{
			RaiseException("invalid special value for the extended boolean class", __LINE__, "StdExtensions::ExtString::Initialise(SpecialValues)", __FILE__, e_DomainError);
		}
	}

	// Retrieve the extended boolean as a native boolean type
	bool ExtBool::GetAsBool() const throw(Exception)
	{
		bool bOut = false;
		switch(iBool)
		{
		case EXTBOOL_FALSE:
			bOut = false;
			break;
		case EXTBOOL_TRUE:
			bOut = true;
			break;
		case EXTBOOL_NA:
		default:
			RaiseException("unable to convert a missing value into a native boolean type", __LINE__, "StdExtensions::ExtBool::GetAsBool()", __FILE__, e_RangeError);
			break;
		}
		return bOut;
	}

	// Inverse the true/false value of the extended boolean type
	ExtBool ExtBool::Inverse() const throw()
	{
		ExtBool cOut;
		switch(iBool)
		{
		case EXTBOOL_FALSE:
			cOut.Initialise(true);
			break;
		case EXTBOOL_TRUE:
			cOut.Initialise(false);
			break;
		case EXTBOOL_NA:
		default:
			cOut.Initialise();
			break;
		}
		return cOut;
	}

	// Testing for equality in the different
	ExtBool ExtBool::Equal(const ExtBool &cObIn) const throw()
	{
		ExtBool cOut;
		if(iBool == EXTBOOL_NA || cObIn.iBool == EXTBOOL_NA)
		{
			cOut.Initialise();
		}
		else
		{
			cOut.Initialise(iBool == cObIn.iBool ? true : false);
		}
		return cOut;
	}

	// Returns true if both input booleans are true
	ExtBool ExtBool::And(const ExtBool &cObIn) const throw()
	{
		ExtBool cOut;
		if(iBool == EXTBOOL_NA || cObIn.iBool == EXTBOOL_NA)
		{
			cOut.Initialise();
		}
		else
		{
			cOut.Initialise(iBool == EXTBOOL_TRUE && cObIn.iBool == EXTBOOL_TRUE ? true : false);
		}
		return cOut;
	}

	// Returns true if either input booleans are true
	ExtBool ExtBool::Or(const ExtBool &cObIn) const throw()
	{
		ExtBool cOut;
		if(iBool == EXTBOOL_NA || cObIn.iBool == EXTBOOL_NA)
		{
			cOut.Initialise();
		}
		else
		{
			cOut.Initialise(iBool == EXTBOOL_TRUE || cObIn.iBool == EXTBOOL_TRUE ? true : false);
		}
		return cOut;
	}

	// Returns true if one of the booleans are true
	ExtBool ExtBool::XOr(const ExtBool &cObIn) const throw()
	{
		ExtBool cOut;
		int iCount = 0;
		if(iBool == EXTBOOL_NA || cObIn.iBool == EXTBOOL_NA)
		{
			cOut.Initialise();
		}
		else
		{
			iCount += (iBool == EXTBOOL_TRUE ? 1 : 0);
			iCount += (cObIn.iBool == EXTBOOL_TRUE ? 1 : 0);
			cOut.Initialise(iCount == 1 ? true : false);
		}
		return cOut;
	}

	// ---------------------------------------
	//  METHODS FOR THE EXTENDED DOUBLE CLASS
	// ---------------------------------------

	// Return the extended double as a native double type
	double ExtDouble::GetAsDouble() const throw(Exception)
	{
		if(eSpec != e_Standard)
		{
			RaiseException("unable to convert a special value to a native double type", __LINE__, "StdExtensions::ExtDouble::GetAsDouble()", __FILE__, e_RangeError);
		}
		return dVal;
	}

	// Return the negative of the calling object
	ExtDouble ExtDouble::Negative() const throw()
	{
		ExtDouble cOut;
		switch(eSpec)
		{
		case e_Standard:			// Negative of a standard value
			cOut.Initialise(-dVal);
			break;
		case e_PositiveInfinity:	// - +Inf = -Inf
			cOut.Initialise(e_NegativeInfinity);
			break;
		case e_NegativeInfinity:	// - -Inf = +Inf
			cOut.Initialise(e_PositiveInfinity);
			break;
		case e_NotANumber:			// - NaN = NaN
			cOut.Initialise(e_NotANumber);
			break;
		case e_MissingValue:		// - NA = NA
		default:
			cOut.Initialise(e_MissingValue);
		}
		return cOut;
	}

	// Return the reciprocal of the calling object
	ExtDouble ExtDouble::Reciprocal() const throw()
	{
		ExtDouble cOut;
		switch(eSpec)
		{
		case e_Standard:
			if(dVal == 0.0)
			{	// 1.0 / 0.0 = +Inf
				cOut.Initialise(e_PositiveInfinity);
			}
			else
			{
				cOut.Initialise(1.0 / dVal);
			}
			break;
		case e_PositiveInfinity:
		case e_NegativeInfinity:
			// 1.0 / +Inf = 0.0
			// 1.0 / -Inf = 0.0
			cOut.Initialise(0.0);
			break;
		case e_NotANumber:
			// 1.0 / NaN = NaN
			cOut.Initialise(e_NotANumber);
			break;
		case e_MissingValue:
		default:
			// 1.0 / NA = NA
			cOut.Initialise(e_MissingValue);
			break;
		}
		return cOut;
	}

	// Addition arithmetic function
	ExtDouble ExtDouble::Add(const ExtDouble &cObIn) const throw()
	{
		ExtDouble cOut;
		switch(eSpec)
		{
		case e_Standard:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				// x + y = x + y
				cOut.Initialise(dVal + cObIn.dVal);
				break;
			case e_PositiveInfinity:
				// x + +Inf = +Inf
				cOut.Initialise(e_PositiveInfinity);
				break;
			case e_NegativeInfinity:
				// x + -Inf = -Inf
				cOut.Initialise(e_NegativeInfinity);
				break;
			case e_NotANumber:
				// x + NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// x + NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_PositiveInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
				// +Inf + x = +Inf
				// +Inf + +Inf = +Inf
				cOut.Initialise(e_PositiveInfinity);
				break;
			case e_NegativeInfinity:
			case e_NotANumber:
				// +Inf + -Inf = NaN
				// +Inf + NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// +Inf + NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_NegativeInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_NegativeInfinity:
				// -Inf + x = -Inf
				// -Inf + -Inf = -Inf
				cOut.Initialise(e_NegativeInfinity);
				break;
			case e_PositiveInfinity:
			case e_NotANumber:
				// -Inf + +Inf = NaN
				// -Inf + NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// -Inf + NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_NotANumber:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
			case e_NegativeInfinity:
			case e_NotANumber:
				// NaN + x = NaN
				// NaN + +Inf = NaN
				// NaN + -Inf = NaN
				// NaN + NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// NaN + NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_MissingValue:
			// NA + x = NA
			// NA + +Inf = NA
			// NA + -Inf = NA
			// NA + NaN = NA
			// NA + NA = NA
			cOut.Initialise(e_MissingValue);
			break;
		}
		return cOut;
	}

	// Multiplication arithmetic function
	ExtDouble ExtDouble::Multiply(const ExtDouble &cObIn) const throw()
	{
		ExtDouble cOut;
		switch(eSpec)
		{
		case e_Standard:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				// x * y = x * y
				cOut.Initialise(dVal * cObIn.dVal);
				break;
			case e_PositiveInfinity:
				if(dVal == 0.0)
				{	// 0.0 * +Inf = NaN
					cOut.Initialise(e_NotANumber);
				}
				else if(dVal < 0.0)
				{	// x * +Inf = -Inf when x < 0.0
					cOut.Initialise(e_NegativeInfinity);
				}
				else
				{	// x * +Inf = +Inf when x > 0.0
					cOut.Initialise(e_PositiveInfinity);
				}
				break;
			case e_NegativeInfinity:
				if(dVal == 0.0)
				{	// 0.0 * -Inf = NaN
					cOut.Initialise(e_NotANumber);
				}
				else if(dVal < 0.0)
				{	// x * -Inf = +Inf if x < 0.0
					cOut.Initialise(e_PositiveInfinity);
				}
				else
				{	// x * -Inf = -Inf if x > 0.0
					cOut.Initialise(e_NegativeInfinity);
				}
				break;
			case e_NotANumber:
				// x * NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// x * NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_PositiveInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				if(cObIn.dVal == 0.0)
				{	// +Inf * 0.0 = NaN
					cOut.Initialise(e_NotANumber);
				}
				else if(cObIn.dVal < 0.0)
				{	// +Inf * x = -Inf if x < 0.0
					cOut.Initialise(e_NegativeInfinity);
				}
				else
				{	// +Inf * x = +Inf if x > 0.0
					cOut.Initialise(e_PositiveInfinity);
				}
				break;
			case e_PositiveInfinity:
				// +Inf * +Inf = +Inf
				cOut.Initialise(e_PositiveInfinity);
				break;
			case e_NegativeInfinity:
				// +Inf * -Inf = -Inf
				cOut.Initialise(e_NegativeInfinity);
				break;
			case e_NotANumber:
				// +Inf * NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// +Inf * NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_NegativeInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				if(cObIn.dVal == 0.0)
				{	// -Inf * 0.0 = NaN
					cOut.Initialise(e_NotANumber);
				}
				else if(cObIn.dVal < 0.0)
				{	// -Inf * x = +Inf (x < 0.0)
					cOut.Initialise(e_PositiveInfinity);
				}
				else
				{	// -Inf * x = -Inf (x > 0.0)
					cOut.Initialise(e_NegativeInfinity);
				}
				break;
			case e_PositiveInfinity:
				// -Inf * +Inf = -Inf
				cOut.Initialise(e_NegativeInfinity);
				break;
			case e_NegativeInfinity:
				// -Inf * -Inf = +Inf
				cOut.Initialise(e_PositiveInfinity);
				break;
			case e_NotANumber:
				// -Inf * NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
				// -Inf * NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_NotANumber:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
			case e_NegativeInfinity:
			case e_NotANumber:
				// NaN * x = NaN
				// NaN * +Inf = NaN
				// NaN * -Inf = NaN
				// NaN * NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// NaN * NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
		case e_MissingValue:
		default:
			// NA * x = NA
			// NA * +Inf = NA
			// NA * -Inf = NA
			// NA * NaN = NA
			// NA * NA = NA
			cOut.Initialise(e_MissingValue);
			break;
		}
		return cOut;
	}

	// Division arithmetic function
	ExtDouble ExtDouble::Divide(const ExtDouble &cObIn) const throw()
	{
		ExtDouble cOut;
		switch(eSpec)
		{
		case e_Standard:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				if(cObIn.dVal != 0.0)
				{	// x / y = x / y (y != 0)
					cOut.Initialise(dVal / cObIn.dVal);
				}
				else
				{
					if(dVal == 0.0)
					{	// 0.0 / 0.0 = NaN
						cOut.Initialise(e_NotANumber);
					}
					else if(dVal > 0.0)
					{	// x / 0.0 = +Inf (x > 0.0)
						cOut.Initialise(e_PositiveInfinity);
					}
					else
					{	// x / 0.0 = -Inf (x < 0.0)
						cOut.Initialise(e_NegativeInfinity);
					}
				}
				break;
			case e_PositiveInfinity:
			case e_NegativeInfinity:
				// x / +Inf = 0.0
				// x / -Inf = 0.0
				cOut.Initialise(0.0);
				break;
			case e_NotANumber:
				// x / NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// x / NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_PositiveInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				if(cObIn.dVal >= 0.0)
				{	// +Inf / x = +Inf (x >= 0.0)
					cOut.Initialise(e_PositiveInfinity);
				}
				else
				{	// +Inf / x = -Inf (x < 0.0)
					cOut.Initialise(e_NegativeInfinity);
				}
				break;
			case e_PositiveInfinity:
			case e_NegativeInfinity:
			case e_NotANumber:
				// +Inf / +Inf = NaN
				// +Inf / -Inf = NaN
				// +Inf / NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// +Inf / NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_NegativeInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				if(cObIn.dVal >= 0.0)
				{	// -Inf / x = -Inf (x >= 0.0)
					cOut.Initialise(e_NegativeInfinity);
				}
				else
				{	// -Inf / x = +Inf (x < 0.0)
					cOut.Initialise(e_PositiveInfinity);
				}
				break;
			case e_PositiveInfinity:
			case e_NegativeInfinity:
			case e_NotANumber:
				// -Inf / +Inf = NaN
				// -Inf / -Inf = NaN
				// -Inf / NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// -Inf / NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_NotANumber:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
			case e_NegativeInfinity:
			case e_NotANumber:
				// NaN / x = NaN
				// NaN / +Inf = NaN
				// NaN / -Inf = NaN
				// NaN / NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// NaN / NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_MissingValue:
		default:
			// NA / x = NA
			// NA / +Inf = NA
			// NA / -Inf = NA
			// NA / NaN = NA
			// NA / NA = NA
			cOut.Initialise(e_MissingValue);
			break;
		}
		return cOut;
	}

	// Function to raise the calling object to a power
	ExtDouble ExtDouble::Pow(const ExtDouble &cObIn) const throw()
	{
		ExtDouble cOut;
		switch(eSpec)
		{
		case e_Standard:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				if(cObIn.dVal == 0.0)
				{	// x^0 = 1
					cOut.Initialise(1.0);
				}
				else if(dVal < 0.0)
				{
					if(cObIn.dVal == double(int(cObIn.dVal)))
					{	// x^y = x^y when x < 0.0 and y is an integer
						cOut.Initialise(pow(dVal, cObIn.dVal));
					}
					else
					{	// x^y = NaN when x < 0.0 and y is not an integer
						cOut.Initialise(e_NotANumber);
					}
				}
				else
				{	// x^y = x^y (x > 0.0)
					cOut.Initialise(pow(dVal, cObIn.dVal));
				}
				break;
			case e_PositiveInfinity:
				if(dVal >= 0.0 && dVal < 1.0)
				{	// x^+Inf = 0 (0.0 <= x < 1.0)
					cOut.Initialise(0.0);
				}
				else if(dVal == 1.0)
				{	// 1.0^+Inf = 1.0
					cOut.Initialise(1.0);
				}
				else if(dVal > 1.0)
				{	// x^+Inf = +Inf (x > 1.0)
					cOut.Initialise(e_PositiveInfinity);
				}
				else
				{	// x^+Inf = NaN (x < 0.0)
					cOut.Initialise(e_NotANumber);
				}
				break;
			case e_NegativeInfinity:
				if(dVal >= 0.0 && dVal < 1.0)
				{	// x^-Inf = +Inf (0.0 <= x < 1.0)
					cOut.Initialise(e_PositiveInfinity);
				}
				else if(dVal == 1.0)
				{	// 1.0^-Inf = 1.0
					cOut.Initialise(1.0);
				}
				else if(dVal > 1.0)
				{	// x^-Inf = 0.0 (x > 1.0)
					cOut.Initialise(0.0);
				}
				else
				{	// x^-Inf = NaN (x < 0.0)
					cOut.Initialise(e_NotANumber);
				}
				break;
			case e_NotANumber:
				// x^NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// x^NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_PositiveInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				if(cObIn.dVal == 0.0)
				{	// +Inf^0 = 1
					cOut.Initialise(1.0);
				}
				else if(cObIn.dVal > 0.0)
				{	// +Inf^x = +Inf (x > 0.0)
					cOut.Initialise(e_PositiveInfinity);
				}
				else
				{	// +Inf^x = 0 (x < 0.0)
					cOut.Initialise(0.0);
				}
				break;
			case e_PositiveInfinity:
				// +Inf^+Inf = +Inf
				cOut.Initialise(e_PositiveInfinity);
				break;
			case e_NegativeInfinity:
				// +Inf^-Inf = 0
				cOut.Initialise(0.0);
				break;
			case e_NotANumber:
				// +Inf^NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// +Inf^NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_NegativeInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				if(cObIn.dVal == 0.0)
				{	// -Inf^0 = 1
					cOut.Initialise(1.0);
				}
				else if(cObIn.dVal == double(int(cObIn.dVal)))
				{
					if(cObIn.dVal < 0.0)
					{	// -Inf^x = 0 when x is an integer and x < 0
						cOut.Initialise(0.0);
					}
					else if(int(cObIn.dVal) % 2 == 0)
					{	// -Inf^x = +Inf when x is a positive even integer
						cOut.Initialise(e_PositiveInfinity);
					}
					else
					{	// -Inf^x = -Inf when x is a positive odd integer
						cOut.Initialise(e_NegativeInfinity);
					}
				}
				else
				{	// -Inf^x = NaN when x is not an integer
					cOut.Initialise(e_NotANumber);
				}
				break;
			case e_PositiveInfinity:
			case e_NegativeInfinity:
			case e_NotANumber:
				// -Inf^+Inf = NaN
				// -Inf^-Inf = NaN
				// -Inf^NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// -Inf^NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_NotANumber:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
			case e_NegativeInfinity:
			case e_NotANumber:
				// NaN^x = NaN
				// NaN^+Inf = NaN
				// NaN^-Inf = NaN
				// NaN^NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// NaN^NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_MissingValue:
		default:
			// NA^x = NA
			// NA^+Inf = NA
			// NA^-Inf = NA
			// NA^NaN = NA
			// NA^NA = NA
			cOut.Initialise();
			break;
		}
		return cOut;
	}

	// Function that returns true if the values of two extended doubles are equal in value
	ExtBool ExtDouble::Equal(const ExtDouble &cObIn) const throw()
	{
		ExtBool cOut;
		// (NA == x) = NA
		// (NaN == x) = NA
		if(eSpec == e_MissingValue || cObIn.eSpec == e_MissingValue ||
			eSpec == e_NotANumber || cObIn.eSpec == e_NotANumber)
		{
			cOut.Initialise();
		}
		else
		{
			cOut.Initialise(dVal == cObIn.dVal && eSpec == cObIn.eSpec);
		}
		return cOut;
	}

	// Function that returns true if the value contained in cObIn is greater than the calling object
	ExtBool ExtDouble::GreaterThan(const ExtDouble &cObIn) const throw()
	{
		ExtBool cOut;
		switch(eSpec)
		{
		case e_Standard:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				// (x > y) = (x > y)
				cOut.Initialise(dVal > cObIn.dVal ? true : false);
				break;
			case e_PositiveInfinity:
				// (x > +Inf) = FALSE
				cOut.Initialise(false);
				break;
			case e_NegativeInfinity:
				// (x > -Inf) = TRUE
				cOut.Initialise(true);
				break;
			case e_NotANumber:
			case e_MissingValue:
			default:
				// (x > NaN) = NA
				// (x > NA) = NA
				cOut.Initialise();
				break;
			}
			break;
		case e_PositiveInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_NegativeInfinity:
				// (+Inf > x) = TRUE
				// (+Inf > -Inf) = TRUE
				cOut.Initialise(true);
				break;
			case e_PositiveInfinity:
				// (+Inf > +Inf) = FALSE
				cOut.Initialise(false);
				break;
			case e_NotANumber:
			case e_MissingValue:
			default:
				// (+Inf > NaN) = NA
				// (+Inf > NA) = NA
				cOut.Initialise();
				break;
			}
			break;
		case e_NegativeInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
			case e_NegativeInfinity:
				// (-Inf > x) = FALSE
				// (-Inf > +Inf) = FALSE
				// (-Inf > -Inf) = FALSE
				cOut.Initialise(false);
				break;
			case e_NotANumber:
			case e_MissingValue:
			default:
				// (-Inf > NaN) = NA
				// (-Inf > NA) = NA
				cOut.Initialise();
				break;
			}
			break;
		case e_NotANumber:
		case e_MissingValue:
		default:
			// (NaN > x) = NA		(NA > x) = NA
			// (NaN > +Inf) = NA	(NA > +Inf) = NA
			// (NaN > -Inf) = NA	(NA > -Inf) = NA
			// (NaN > NaN) = NA		(NA > NaN) = NA
			// (NaN > NA) = NA		(NA > NA) = NA
			cOut.Initialise();
			break;
		}
		return cOut;
	}

	// Function that returns true if the value contained in cObIn is greater than or equal to the calling object
	ExtBool ExtDouble::GreaterThanOrEqual(const ExtDouble &cObIn) const throw()
	{
		ExtBool cOut;
		switch(eSpec)
		{
		case e_Standard:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				// (x >= y) = (x >= y)
				cOut.Initialise(dVal >= cObIn.dVal ? true : false);
				break;
			case e_PositiveInfinity:
				// (x >= +Inf) = FALSE
				cOut.Initialise(false);
				break;
			case e_NegativeInfinity:
				// (x >= -Inf) = TRUE
				cOut.Initialise(true);
				break;
			case e_NotANumber:
			case e_MissingValue:
			default:
				// (x >= NaN) = NA
				// (x >= NA) = NA
				cOut.Initialise();
				break;
			}
			break;
		case e_PositiveInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
			case e_NegativeInfinity:
				// (+Inf >= x) = TRUE
				// (+Inf >= +Inf) = TRUE
				// (+Inf >= -Inf) = TRUE
				cOut.Initialise(true);
				break;
			case e_NotANumber:
			case e_MissingValue:
			default:
				// (+Inf >= NaN) = NA
				// (+Inf >= NA) = NA
				cOut.Initialise();
				break;
			}
			break;
		case e_NegativeInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
				// (-Inf >= x) = FALSE
				// (-Inf >= +Inf) = FALSE
				cOut.Initialise(false);
				break;
			case e_NegativeInfinity:
				// (-Inf >= -Inf) = TRUE
				cOut.Initialise(true);
				break;
			case e_NotANumber:
			case e_MissingValue:
			default:
				// (-Inf >= NaN) = NA
				// (-Inf >= NA) = NA
				cOut.Initialise();
				break;
			}
			break;
		case e_NotANumber:
		case e_MissingValue:
		default:
			// (NaN > x) = NA		(NA > x) = NA
			// (NaN > +Inf) = NA	(NA > +Inf) = NA
			// (NaN > -Inf) = NA	(NA > -Inf) = NA
			// (NaN > NaN) = NA		(NA > NaN) = NA
			// (NaN > NA) = NA		(NA > NA) = NA
			cOut.Initialise();
			break;
		}
		return cOut;
	}

	// ----------------------------------------
	//  METHODS FOR THE EXTENDED INTEGER CLASS
	// ----------------------------------------

	// Return the extended integer as a native integer type
	int ExtInteger::GetAsInteger() const throw(Exception)
	{
		if(eSpec != e_Standard)
		{
			RaiseException("unable to convert a special value to a native integer type", __LINE__, "StdExtensions::ExtInteger::GetAsInteger()", __FILE__, e_RangeError);
		}
		return iVal;
	}

	// Return the negative of the calling object
	ExtInteger ExtInteger::Negative() const throw()
	{
		ExtInteger cOut;
		switch(eSpec)
		{
		case e_Standard:			// Negative of a standard value
			cOut.Initialise(-iVal);
			break;
		case e_PositiveInfinity:	// - +Inf = -Inf
			cOut.Initialise(e_NegativeInfinity);
			break;
		case e_NegativeInfinity:	// - -Inf = +Inf
			cOut.Initialise(e_PositiveInfinity);
			break;
		case e_NotANumber:			// - NaN = NaN
			cOut.Initialise(e_NotANumber);
			break;
		case e_MissingValue:		// - NA = NA
		default:
			cOut.Initialise(e_MissingValue);
		}
		return cOut;
	}

	// Addition arithmetic function
	ExtInteger ExtInteger::Add(const ExtInteger &cObIn) const throw()
	{
		ExtInteger cOut;
		switch(eSpec)
		{
		case e_Standard:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				// x + y = x + y
				cOut.Initialise(iVal + cObIn.iVal);
				break;
			case e_PositiveInfinity:
				// x + +Inf = +Inf
				cOut.Initialise(e_PositiveInfinity);
				break;
			case e_NegativeInfinity:
				// x + -Inf = -Inf
				cOut.Initialise(e_NegativeInfinity);
				break;
			case e_NotANumber:
				// x + NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// x + NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_PositiveInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
				// +Inf + x = +Inf
				// +Inf + +Inf = +Inf
				cOut.Initialise(e_PositiveInfinity);
				break;
			case e_NegativeInfinity:
			case e_NotANumber:
				// +Inf + -Inf = NaN
				// +Inf + NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// +Inf + NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_NegativeInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_NegativeInfinity:
				// -Inf + x = -Inf
				// -Inf + -Inf = -Inf
				cOut.Initialise(e_NegativeInfinity);
				break;
			case e_PositiveInfinity:
			case e_NotANumber:
				// -Inf + +Inf = NaN
				// -Inf + NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// -Inf + NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_NotANumber:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
			case e_NegativeInfinity:
			case e_NotANumber:
				// NaN + x = NaN
				// NaN + +Inf = NaN
				// NaN + -Inf = NaN
				// NaN + NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// NaN + NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_MissingValue:
			// NA + x = NA
			// NA + +Inf = NA
			// NA + -Inf = NA
			// NA + NaN = NA
			// NA + NA = NA
			cOut.Initialise(e_MissingValue);
			break;
		}
		return cOut;
	}

	// Multiplication arithmetic function
	ExtInteger ExtInteger::Multiply(const ExtInteger &cObIn) const throw()
	{
		ExtInteger cOut;
		switch(eSpec)
		{
		case e_Standard:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				// x * y = x * y
				cOut.Initialise(iVal * cObIn.iVal);
				break;
			case e_PositiveInfinity:
				if(iVal == 0)
				{	// 0 * +Inf = NaN
					cOut.Initialise(e_NotANumber);
				}
				else if(iVal < 0)
				{	// x * +Inf = -Inf when x < 0
					cOut.Initialise(e_NegativeInfinity);
				}
				else
				{	// x * +Inf = +Inf when x > 0
					cOut.Initialise(e_PositiveInfinity);
				}
				break;
			case e_NegativeInfinity:
				if(iVal == 0)
				{	// 0 * -Inf = NaN
					cOut.Initialise(e_NotANumber);
				}
				else if(iVal < 0)
				{	// x * -Inf = +Inf if x < 0
					cOut.Initialise(e_PositiveInfinity);
				}
				else
				{	// x * -Inf = -Inf if x > 0
					cOut.Initialise(e_NegativeInfinity);
				}
				break;
			case e_NotANumber:
				// x * NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// x * NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_PositiveInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				if(cObIn.iVal == 0)
				{	// +Inf * 0 = NaN
					cOut.Initialise(e_NotANumber);
				}
				else if(cObIn.iVal < 0)
				{	// +Inf * x = -Inf if x < 0
					cOut.Initialise(e_NegativeInfinity);
				}
				else
				{	// +Inf * x = +Inf if x > 0
					cOut.Initialise(e_PositiveInfinity);
				}
				break;
			case e_PositiveInfinity:
				// +Inf * +Inf = +Inf
				cOut.Initialise(e_PositiveInfinity);
				break;
			case e_NegativeInfinity:
				// +Inf * -Inf = -Inf
				cOut.Initialise(e_NegativeInfinity);
				break;
			case e_NotANumber:
				// +Inf * NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// +Inf * NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_NegativeInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				if(cObIn.iVal == 0)
				{	// -Inf * 0 = NaN
					cOut.Initialise(e_NotANumber);
				}
				else if(cObIn.iVal < 0)
				{	// -Inf * x = +Inf (x < 0)
					cOut.Initialise(e_PositiveInfinity);
				}
				else
				{	// -Inf * x = -Inf (x > 0)
					cOut.Initialise(e_NegativeInfinity);
				}
				break;
			case e_PositiveInfinity:
				// -Inf * +Inf = -Inf
				cOut.Initialise(e_NegativeInfinity);
				break;
			case e_NegativeInfinity:
				// -Inf * -Inf = +Inf
				cOut.Initialise(e_PositiveInfinity);
				break;
			case e_NotANumber:
				// -Inf * NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// -Inf * NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_NotANumber:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
			case e_NegativeInfinity:
			case e_NotANumber:
				// NaN * x = NaN
				// NaN * +Inf = NaN
				// NaN * -Inf = NaN
				// NaN * NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// NaN * NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
		case e_MissingValue:
		default:
			// NA * x = NA
			// NA * +Inf = NA
			// NA * -Inf = NA
			// NA * NaN = NA
			// NA * NA = NA
			cOut.Initialise(e_MissingValue);
			break;
		}
		return cOut;
	}

	// Division arithmetic function
	ExtInteger ExtInteger::Divide(const ExtInteger &cObIn) const throw()
	{
		ExtInteger cOut;
		switch(eSpec)
		{
		case e_Standard:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				if(cObIn.iVal != 0)
				{	// x / y = x / y (y != 0)
					cOut.Initialise(iVal / cObIn.iVal);
				}
				else
				{
					if(iVal == 0)
					{	// 0 / 0 = NaN
						cOut.Initialise(e_NotANumber);
					}
					else if(iVal > 0)
					{	// x / 0 = +Inf (x > 0)
						cOut.Initialise(e_PositiveInfinity);
					}
					else
					{	// x / 0 = -Inf (x < 0)
						cOut.Initialise(e_NegativeInfinity);
					}
				}
				break;
			case e_PositiveInfinity:
			case e_NegativeInfinity:
				// x / +Inf = 0
				// x / -Inf = 0
				cOut.Initialise(0);
				break;
			case e_NotANumber:
				// x / NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// x / NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_PositiveInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				if(cObIn.iVal >= 0)
				{	// +Inf / x = +Inf (x >= 0)
					cOut.Initialise(e_PositiveInfinity);
				}
				else
				{	// +Inf / x = -Inf (x < 0)
					cOut.Initialise(e_NegativeInfinity);
				}
				break;
			case e_PositiveInfinity:
			case e_NegativeInfinity:
			case e_NotANumber:
				// +Inf / +Inf = NaN
				// +Inf / -Inf = NaN
				// +Inf / NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// +Inf / NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_NegativeInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				if(cObIn.iVal >= 0)
				{	// -Inf / x = -Inf (x >= 0)
					cOut.Initialise(e_NegativeInfinity);
				}
				else
				{	// -Inf / x = +Inf (x < 0)
					cOut.Initialise(e_PositiveInfinity);
				}
				break;
			case e_PositiveInfinity:
			case e_NegativeInfinity:
			case e_NotANumber:
				// -Inf / +Inf = NaN
				// -Inf / -Inf = NaN
				// -Inf / NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// -Inf / NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_NotANumber:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
			case e_NegativeInfinity:
			case e_NotANumber:
				// NaN / x = NaN
				// NaN / +Inf = NaN
				// NaN / -Inf = NaN
				// NaN / NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// NaN / NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_MissingValue:
		default:
			// NA / x = NA
			// NA / +Inf = NA
			// NA / -Inf = NA
			// NA / NaN = NA
			// NA / NA = NA
			cOut.Initialise(e_MissingValue);
			break;
		}
		return cOut;
	}

	// Modulo arithmetic function
	ExtInteger ExtInteger::Modulo(const ExtInteger &cObIn) const throw()
	{
		ExtInteger cOut;
		switch(eSpec)
		{
		case e_Standard:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				if(cObIn.iVal != 0)
				{	// x % y = x % y (y != 0)
					cOut.Initialise(iVal % cObIn.iVal);
				}
				else
				{	// x % 0 = NaN
					cOut.Initialise(e_NotANumber);
				}
				break;
			case e_PositiveInfinity:
			case e_NegativeInfinity:
			case e_NotANumber:
				// x % +Inf = NaN
				// x % -Inf = NaN
				// x % NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// x % NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_PositiveInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
			case e_NegativeInfinity:
			case e_NotANumber:
				// +Inf % x = NaN
				// +Inf % +Inf = NaN
				// +Inf % -Inf = NaN
				// +Inf % NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// +Inf % NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_NegativeInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
			case e_NegativeInfinity:
			case e_NotANumber:
				// -Inf % x = NaN
				// -Inf % +Inf = NaN
				// -Inf % -Inf = NaN
				// -Inf % NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// -Inf % NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_NotANumber:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
			case e_NegativeInfinity:
			case e_NotANumber:
				// NaN % x = NaN
				// NaN % +Inf = NaN
				// NaN % -Inf = NaN
				// NaN % NaN = NaN
				cOut.Initialise(e_NotANumber);
				break;
			case e_MissingValue:
			default:
				// NaN % NA = NA
				cOut.Initialise(e_MissingValue);
				break;
			}
			break;
		case e_MissingValue:
		default:
			// NA % x = NA
			// NA % +Inf = NA
			// NA % -Inf = NA
			// NA % NaN = NA
			// NA % NA = NA
			cOut.Initialise(e_MissingValue);
			break;
		}
		return cOut;
	}

	// Function that returns true if the values of two extended doubles are equal in value
	ExtBool ExtInteger::Equal(const ExtInteger &cObIn) const throw()
	{
		ExtBool cOut;
		// (NA == x) = NA
		// (NaN == x) = NA
		if(eSpec == e_MissingValue || cObIn.eSpec == e_MissingValue ||
			eSpec == e_NotANumber || cObIn.eSpec == e_NotANumber)
		{
			cOut.Initialise();
		}
		else
		{
			cOut.Initialise(iVal == cObIn.iVal && eSpec == cObIn.eSpec);
		}
		return cOut;
	}

	// Function that returns true if the value contained in cObIn is greater than the calling object
	ExtBool ExtInteger::GreaterThan(const ExtInteger &cObIn) const throw()
	{
		ExtBool cOut;
		switch(eSpec)
		{
		case e_Standard:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				// (x > y) = (x > y)
				cOut.Initialise(iVal > cObIn.iVal ? true : false);
				break;
			case e_PositiveInfinity:
				// (x > +Inf) = FALSE
				cOut.Initialise(false);
				break;
			case e_NegativeInfinity:
				// (x > -Inf) = TRUE
				cOut.Initialise(true);
				break;
			case e_NotANumber:
			case e_MissingValue:
			default:
				// (x > NaN) = NA
				// (x > NA) = NA
				cOut.Initialise();
				break;
			}
			break;
		case e_PositiveInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_NegativeInfinity:
				// (+Inf > x) = TRUE
				// (+Inf > -Inf) = TRUE
				cOut.Initialise(true);
				break;
			case e_PositiveInfinity:
				// (+Inf > +Inf) = FALSE
				cOut.Initialise(false);
				break;
			case e_NotANumber:
			case e_MissingValue:
			default:
				// (+Inf > NaN) = NA
				// (+Inf > NA) = NA
				cOut.Initialise();
				break;
			}
			break;
		case e_NegativeInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
			case e_NegativeInfinity:
				// (-Inf > x) = FALSE
				// (-Inf > +Inf) = FALSE
				// (-Inf > -Inf) = FALSE
				cOut.Initialise(false);
				break;
			case e_NotANumber:
			case e_MissingValue:
			default:
				// (-Inf > NaN) = NA
				// (-Inf > NA) = NA
				cOut.Initialise();
				break;
			}
			break;
		case e_NotANumber:
		case e_MissingValue:
		default:
			// (NaN > x) = NA		(NA > x) = NA
			// (NaN > +Inf) = NA	(NA > +Inf) = NA
			// (NaN > -Inf) = NA	(NA > -Inf) = NA
			// (NaN > NaN) = NA		(NA > NaN) = NA
			// (NaN > NA) = NA		(NA > NA) = NA
			cOut.Initialise();
			break;
		}
		return cOut;
	}

	// Function that returns true if the value contained in cObIn is greater than or equal to the calling object
	ExtBool ExtInteger::GreaterThanOrEqual(const ExtInteger &cObIn) const throw()
	{
		ExtBool cOut;
		switch(eSpec)
		{
		case e_Standard:
			switch(cObIn.eSpec)
			{
			case e_Standard:
				// (x >= y) = (x >= y)
				cOut.Initialise(iVal >= cObIn.iVal ? true : false);
				break;
			case e_PositiveInfinity:
				// (x >= +Inf) = FALSE
				cOut.Initialise(false);
				break;
			case e_NegativeInfinity:
				// (x >= -Inf) = TRUE
				cOut.Initialise(true);
				break;
			case e_NotANumber:
			case e_MissingValue:
			default:
				// (x >= NaN) = NA
				// (x >= NA) = NA
				cOut.Initialise();
				break;
			}
			break;
		case e_PositiveInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
			case e_NegativeInfinity:
				// (+Inf >= x) = TRUE
				// (+Inf >= +Inf) = TRUE
				// (+Inf >= -Inf) = TRUE
				cOut.Initialise(true);
				break;
			case e_NotANumber:
			case e_MissingValue:
			default:
				// (+Inf >= NaN) = NA
				// (+Inf >= NA) = NA
				cOut.Initialise();
				break;
			}
			break;
		case e_NegativeInfinity:
			switch(cObIn.eSpec)
			{
			case e_Standard:
			case e_PositiveInfinity:
				// (-Inf >= x) = FALSE
				// (-Inf >= +Inf) = FALSE
				cOut.Initialise(false);
				break;
			case e_NegativeInfinity:
				// (-Inf >= -Inf) = TRUE
				cOut.Initialise(true);
				break;
			case e_NotANumber:
			case e_MissingValue:
			default:
				// (-Inf >= NaN) = NA
				// (-Inf >= NA) = NA
				cOut.Initialise();
				break;
			}
			break;
		case e_NotANumber:
		case e_MissingValue:
		default:
			// (NaN > x) = NA		(NA > x) = NA
			// (NaN > +Inf) = NA	(NA > +Inf) = NA
			// (NaN > -Inf) = NA	(NA > -Inf) = NA
			// (NaN > NaN) = NA		(NA > NaN) = NA
			// (NaN > NA) = NA		(NA > NA) = NA
			cOut.Initialise();
			break;
		}
		return cOut;
	}

	// ----------------------------------------
	//  METHODS FOR THE EXTENDED COMPLEX CLASS
	// ----------------------------------------

	// Multiplication arithmetic function
	ExtComplex ExtComplex::Multiply(const ExtComplex &cObIn) const throw()
	{
		return ExtComplex(
			cReal.Multiply(cObIn.cReal).Subtract(cImag.Multiply(cObIn.cImag)),
			cReal.Multiply(cObIn.cImag).Add(cImag.Multiply(cObIn.cReal)));
	}

	// Division arithmetic function
	ExtComplex ExtComplex::Divide(const ExtComplex &cObIn) const throw()
	{
		return ExtComplex(
			cReal.Multiply(cObIn.cReal).Add(cImag.Multiply(cObIn.cImag)).Divide(
				cObIn.cReal.Pow(2.0).Add(cObIn.cImag.Pow(2.0))),
			cImag.Multiply(cObIn.cReal).Subtract(cReal.Multiply(cObIn.cImag)).Divide(
				cObIn.cReal.Pow(2.0).Add(cObIn.cImag.Pow(2.0))));
	}

	// Testing for equality
	ExtBool ExtComplex::Equal(const ExtComplex &cObIn) const throw()
	{
		return cReal.Equal(cObIn.cReal).And(cImag.Equal(cObIn.cImag));
	}
}

//Return warnings to default upon compilation with Visual C++ compiler
#ifdef _MSC_VER
	#pragma warning( default : 4290 )
#endif
