#ifndef __LATTICEDISPERSAL_H__
#define __LATTICEDISPERSAL_H__

#include "RInterface.h"

//If the file is being compiled with a Visual C++ compiler then set it to be included once in the program only
//and to temporarily disable exception specification warnings generated as a result of non-standard behaviour
#ifdef _MSC_VER
	#pragma once
	#pragma warning( disable : 4290 )
#endif

namespace LatticeDispersal
{
	// Simple enumeration to hold the different boundary condition types
	enum BoundaryConditions
	{
		e_Restricting,	// Restricting boundary conditions
		e_Absorbing,	// Absorbing boundary conditions
		e_Periodic		// Periodic boundary conditions
	};

	// Simple enumber to hold the different approximation methods
	enum ApproxMethod
	{
		e_CC,			// Centroid-to-centroid dispersal
		e_CA,			// Centroid-to-Area dispersal
		e_AC,			// Area-to-centroid dispersal
		e_AA			// Area-to-area dispersal
	};
}

// Function to create a set of grid cells based on a set of x and y coordinates
//
//	x					-	Vector of x-coordinates of the grid cells
//	y					-	Vector of y-coordinates of the grid cells
//
#ifdef _MSC_VER
	extern "C" _declspec(dllexport) SEXP MakeGridFromCoords(SEXP pArgs);
#else
	extern "C" SEXP MakeGridFromCoords(SEXP pArgs);
#endif

// Function to evaluate the error function
//
//	x					-	Vector of doubles at which the error function evaluation is required
//
#ifdef _MSC_VER
	extern "C" _declspec(dllexport) SEXP Erf(SEXP pArgs);
#else
	extern "C" SEXP Erf(SEXP pArgs);
#endif

// Function to perform simple multidimensional Monte Carlo integration
//
//	func				-	Integrand function
//	lower				-	Lower limit of integration
//	upper				-	Upper limit of integration
//	initial.step		-	Number of runs to do in the first step
//	step.size			-	Number of runs to do in one time step
//	max.rel.var			-	Maximum relative variance before stopping MC integration
//	...					-	Extra parameters to pass to the integrand function
//
#ifdef _MSC_VER
	extern "C" _declspec(dllexport) SEXP MCIntegration(SEXP pArgs);
#else
	extern "C" SEXP MCIntegration(SEXP pArgs);
#endif

// Function to calculate cell-to-cell transition probabilities based on different approximations
// to continuous Gaussian dispersal.  We use the two dimension Gaussian function used in Clark et
// al. 1999
//
//	x1					-	Vector of lower boundary coordinates on the x-dimension
//	x2					-	Vector of upper boundary coordinates on the x-dimension
//	y1					-	Vector of lower boundary coordinates on the y-dimension
//	y2					-	Vector of upper boundary coordinates on the y-dimension
//	alpha				-	Scalar value for the shape parameter of the Gaussian distribution
//	approx.method		-	Single string describing the approximation method to employ.  Must
//							be one of the following values:
//								"CC" - Centroid-to-centroid dispersal
//								"CA" - Centroid-to-area dispersal
//								"AC" - Area-to-centroid dispersal
//								"AA" - Area-to-area dispersal
//	boundary			-	Single string describing the boundary condition to apply.  Must be
//							one of the following values:
//								"restricting" - Restricting boundary conditions
//								"absorbing" - Absorbing boundary conditions
//								"periodic" - Periodic boundary conditions
//	max.prob			-	The maximum probability of any extra arena length
//							before stopping condition is met when calculating
//							torus boundary conditions
//
#ifdef _MSC_VER
	extern "C" _declspec(dllexport) SEXP GaussianLatticeTransition(SEXP pArgs);
#else
	extern "C" SEXP GaussianLatticeTransition(SEXP pArgs);
#endif

// Function to calculate cell-to-cell transition probabilities based on different approximations
// to continuous dispersal.  This function calculates transition probabilities for uneven grid.
//
//	x1					-	Vector of lower boundary coordinates on the x-dimension
//	x2					-	Vector of upper boundary coordinates on the x-dimension
//	y1					-	Vector of lower boundary coordinates on the y-dimension
//	y2					-	Vector of upper boundary coordinates on the y-dimension
//	func				-	Dispersal function for the description of the two dimensional dispersal kernel
//	approx.method		-	Single string describing the approximation method to employ.  Must
//							be one of the following values:
//								"CC" - Centroid-to-centroid dispersal
//								"CA" - Centroid-to-area dispersal
//								"AC" - Area-to-centroid dispersal
//								"AA" - Area-to-area dispersal
//	boundary			-	Single string describing the boundary condition to apply.  Must be
//							one of the following values:
//								"restricting" - Restricting boundary conditions
//								"absorbing" - Absorbing boundary conditions
//								"periodic" - Periodic boundary conditions
//	max.prob			-	The maximum probability of any extra arena length
//							before stopping condition is met when calculating
//							torus boundary conditions
//	initial.step		-	MC integration parameter: number of runs to do in the first step
//	step.size			-	MC integration parameter: number of runs to complete in each time step
//	max.rel.var			-	MC integration parameter: maximum relative variance before stopping MC integration
//	params				-	Parameter vector used in dispersal calculation
//	...					-	Extra parameters to pass to the dispersal function
//
#ifdef _MSC_VER
	extern "C" _declspec(dllexport) SEXP LatticeTransition(SEXP pArgs);
#else
	extern "C" SEXP LatticeTransition(SEXP pArgs);
#endif

//Return warnings to default upon compilation with Visual C++ compiler
#ifdef _MSC_VER
	#pragma warning( default : 4290 )
#endif

#endif
