// Inclusion of this file requires linking to the R libraries

#ifndef __RINTERFACE_H__
#define __RINTERFACE_H__

#include <cstdio>
#include <climits>
#include <cfloat>
#include <cerrno>
#include <cstdarg>
#include <typeinfo>
#include <cstdlib>

// Stop R libraries from trying to include C headers that clash with C++ ones
#define NO_C_HEADERS
// Stop R libraries from defining re-mapping macros that will clash with other libraries
#define R_NO_REMAP
// If ERROR is defined at this point then it will clash with the ERROR macro used in R
#ifdef ERROR
	#undef ERROR
#endif

#include "R.h"
#include "Rdefines.h"
#include "ExtString.h"
#include "Random.h"

//If the file is being compiled with a Visual C++ compiler then set it to be included once in the program only
//and to temporarily disable exception specification warnings generated as a result of non-standard behaviour
#ifdef _MSC_VER
	#pragma once
	#pragma warning( disable : 4290 )
#endif

namespace RInterface
{	// ---------------------------
	//  R OBJECT TYPE ENUMERATION
	// ---------------------------
	// Enumeration to hold object types
	enum RObjectType
	{
		e_NULL,		// Null data type
		e_LIST,		// List data type
		e_INTVEC,	// Integer vector
		e_DBLVEC,	// Double vector
		e_CPXVEC,	// Complex vector
		e_STRVEC,	// String vector
		e_BOOLVEC,	// Boolean vector
		e_FUNC,		// Function
		e_ENV		// Environment 
	};

	// Function to find an R object type
	RObjectType CalculateObjectType(SEXP pRObIn) throw(StdExtensions::Exception);

	// PROTECTION FUNCTIONS

	// Function to protect and count
	void ProtectAndCount(SEXP pRObIn, size_t &ulIter) throw();
	// Function to unprotect and recount
	void UnprotectAndCount(size_t ulNum, size_t &ulIter) throw();
	// Function to unprotect all
	void UnprotectAll(size_t &ulIter) throw();

	// Retrieve a parameter from a list
	SEXP GetParameterItem(SEXP pParamList, const StdExtensions::ExtString &cObIn) throw(StdExtensions::Exception);
	SEXP GetParameterItem(SEXP pParamList, const char *pStrIn) throw(StdExtensions::Exception);

	// Write a standard exception to the R console
	void WriteRException(const StdExtensions::Exception &cObIn) throw();

	// ---------------------------------
	//  R RANDOM NUMBER GENERATOR CLASS
	// ---------------------------------
	// Class to draw random variables using R's random number generator
	class RRandom : public StdExtensions::Random
	{
	private:
		// Static variable to count the current number of random number generators
		static int iNumGenerators;
		void AddGenerator() throw();		// Add a generator object
		void RemoveGenerator() throw();		// Remove a generator object
	protected:
		// Set the uniform random number generator
		virtual double UniformGen() throw();
	public:
		RRandom() throw() : Random() {AddGenerator();};		// Constructor - add generator object
		virtual ~RRandom() throw() {RemoveGenerator();};	// Destructor - delete generator object
		// Set the seed function - does nothing as R random number generator cannot
		// be accessed using C functions
		virtual void Seed(int iSeedVal) throw() {iSeedVal++;};
	};

	// ------------------------
	//  R OUTPUT STREAM OBJECT
	// ------------------------
	// Class to print outputs to the R console
	class RConsoleOut
	{
	private:
		bool bAutoFlush;													// Boolean denoting whether flush is made after every output
		void Write(int iNum) const throw();									// Output integer
		void Write(double dNum) const throw();								// Output double
		void Write(const char *pString) const throw();						// Output string
		void Write(bool bOut) const throw();								// Output boolean
		void Write(const StdExtensions::ExtInteger &cObIn) const throw();	// Output extended integer
		void Write(const StdExtensions::ExtDouble &cObIn) const throw();	// Output extended double
		void Write(const StdExtensions::ExtComplex &cObIn) const throw();	// Output extended complex number
		void Write(const StdExtensions::ExtString &cObIn) const throw();	// Output extended string
		void Write(const StdExtensions::ExtBool &cObIn) const throw();		// Output extended boolean
	public:
		// Constructor and destructor
		RConsoleOut() throw() : bAutoFlush(false) {};
		~RConsoleOut() throw() {Flush();};
		// Function to flush the buffer to the output
		void Flush() const throw();
		// Function to set or unset auto flush
		void SetAutoFlush(bool bSet) throw() {bAutoFlush = bSet;};
		// Overloaded stream operators to write outputs
		friend RConsoleOut& operator << (RConsoleOut &cStream, int iNum) throw() {cStream.Write(iNum); return cStream;};
		friend RConsoleOut& operator << (RConsoleOut &cStream, double dNum) throw() {cStream.Write(dNum); return cStream;};
		friend RConsoleOut& operator << (RConsoleOut &cStream, const char *pString) throw() {cStream.Write(pString); return cStream;};
		friend RConsoleOut& operator << (RConsoleOut &cStream, bool bOut) throw() {cStream.Write(bOut); return cStream;};
		friend RConsoleOut& operator << (RConsoleOut &cStream, StdExtensions::ExtInteger &cObIn) throw() {cStream.Write(cObIn); return cStream;};
		friend RConsoleOut& operator << (RConsoleOut &cStream, StdExtensions::ExtDouble &cObIn) throw() {cStream.Write(cObIn); return cStream;};
		friend RConsoleOut& operator << (RConsoleOut &cStream, StdExtensions::ExtComplex &cObIn) throw() {cStream.Write(cObIn); return cStream;};
		friend RConsoleOut& operator << (RConsoleOut &cStream, StdExtensions::ExtString &cObIn) throw() {cStream.Write(cObIn); return cStream;};
		friend RConsoleOut& operator << (RConsoleOut &cStream, StdExtensions::ExtBool &cObIn) throw() {cStream.Write(cObIn); return cStream;};
	};

	// -----------------------
	//  PROTECTED STACK CLASS
	// -----------------------
	// Class to automatically handle protection and unprotection of SEXP objects
	class RProtectedStack
	{
	public:
		// ------------------------------
		//  PROTECTED STACK LINKER CLASS
		// ------------------------------
		// Class to provide links to protected stack SEXP objects
		class RProtectedStackLinker
		{
			friend class RProtectedStack;
		private:
			SEXP pObject;					// Linked SEXP objects
			PROTECT_INDEX iIndx;			// Protection index
			RProtectedStackLinker *pNext;	// The next SEXP object linker in the list
			// Initialise the protected functions
			void Initialise(SEXP pObIn, RProtectedStackLinker *pNextIn) throw();
		public:
			// Null initialisation functions
			RProtectedStackLinker() throw() : pObject(R_NilValue), iIndx(0), pNext(NULL) {};
			void Initialise() throw();
			~RProtectedStackLinker() throw() {Initialise();};
			// Get the index of the object
			PROTECT_INDEX GetIndex() const throw() {return iIndx;};
		};
	private:
		RProtectedStackLinker *pStart;			// The head of the stack
	public:
		// Null initialisation functions
		RProtectedStack() throw() : pStart(NULL) {};
		~RProtectedStack() throw() {UnprotectAll();};
		// PROTECT / UNPROTECT FUNCTIONS
		PROTECT_INDEX Protect(SEXP pRObIn) throw(StdExtensions::Exception);	// Protect the object pointed to by pRObIn
		void UnprotectAll() throw();										// Unprotect all objects in the stack
		void Unprotect(size_t ulIter) throw();								// Unprotect the last ulIter objects added to the stack
	};

	// -----------------------------
	//  DIMENSION INFORMATION CLASS
	// -----------------------------
	// Class to hold information on attributes of the dimensionality of the R object.
	// Includes information on dimension names.  Abstract base class
	class RDimensionInfo
	{
	protected:
		size_t ulNumDims;						// Number of dimensions
		int *pDimSize;							// The extent of each dimension
		StdExtensions::ExtString **pDimNames;	// Names given to dimension coordinates
		// Copy dimension information from another object
		void CopyDimensionStructure(const RDimensionInfo *pObIn) throw(StdExtensions::Exception);
	public:
		RDimensionInfo() throw() : ulNumDims(0), pDimSize(NULL), pDimNames(NULL) {SetDimensionStructure();};
		// Pure virtual function to calculate the length of the object
		virtual size_t Length() const throw() = 0;
		// Functions to convert a location on a flat array to a set of dimensional coordinates
		// and vice versa
		void ConvertIterToDimLoc(int iIter, int *pCoordsOut) const throw(StdExtensions::Exception);
		int ConvertDimLocToIter(const int *pCoordsIn) const throw(StdExtensions::Exception);
		// Set dimension information
		void SetDimensionStructure() throw();
		void SetDimensionStructure(size_t ulDimNumIn, const int *pDimIn, StdExtensions::ExtString **pNamesIn = NULL) throw(StdExtensions::Exception);
		// Get dimension information
		//		retrieve the number of dimensions
		size_t GetNumberDimensions() const throw() {return ulNumDims;};
		//		retrieve a dimension extent
		size_t GetDimensionExtent(size_t ulDimIter) const throw(StdExtensions::Exception);
		// Import dimension information from a SEXP object
		void ImportDimensionStructure(SEXP pRObIn) throw(StdExtensions::Exception);
		// Export dimension names as a SEXP object
		SEXP ExportDimensionNamesAsSEXP() const throw(StdExtensions::Exception);
		// Export dimension extents as a SEXP object
		SEXP ExportDimensionExtentsAsSEXP() const throw();
		// Destructor function
		virtual ~RDimensionInfo() throw() {SetDimensionStructure();};
	};

	// ---------------------
	//  NULL R OBJECT CLASS
	// ---------------------
	// Base class that forms part of other complex R objects.  Without inheritance
	// this class can only hold NULL data objects.
	class RObject
	{
	private:
		RObjectType eObType;		// Object type
	protected:
		// Functions to set the object type
		RObject(RObjectType eObTypeIn) throw() : eObType(eObTypeIn) {};
		void SetObjectType(RObjectType eObTypeIn) throw() {eObType = eObTypeIn;};
	public:
		// Null initialisation functions
		RObject() throw() : eObType(e_NULL) {};
		// Return the object type
		RObjectType GetObjectType() const throw() {return eObType;};
		// Import information from an R SEXP object
		virtual void ImportFromSEXP(SEXP pRObIn) throw(StdExtensions::Exception) {pRObIn = R_NilValue;};
		// Export the object as an R SEXP object
		virtual SEXP ExportAsSEXP() const throw(StdExtensions::Exception) {return R_NilValue;};
		// Virtual destructor function
		virtual ~RObject() throw() {};
	};
	
	// A null object instantiation
	const RObject RNULL_OBJECT;

	// ------------------
	//  R INTEGER VECTOR
	// ------------------
	// Class to hold R integer vector objects.
	class RIntegerVector : public RObject, public RDimensionInfo
	{
	private:
		StdExtensions::ExtInteger *pVector;			// Array of extended integers
		size_t ulVecSize;							// Size of array
	public:
		// Null initialisation functions
		RIntegerVector() throw() : RObject(e_INTVEC), RDimensionInfo(), pVector(NULL), ulVecSize(0) {};
		void Initialise() throw();
		virtual ~RIntegerVector() throw() {Initialise();};
		// Copy initialisation functions
		RIntegerVector(const RIntegerVector &cObIn) throw(StdExtensions::Exception) : RObject(e_INTVEC), RDimensionInfo(), pVector(NULL), ulVecSize(0) {Initialise(cObIn);};
		void Initialise(const RIntegerVector &cObIn) throw(StdExtensions::Exception);
		RIntegerVector& operator = (const RIntegerVector &cObIn) throw(StdExtensions::Exception) {Initialise(cObIn); return *this;};
		// Parameterised initialisation functions
		RIntegerVector(size_t ulLengthIn) throw(StdExtensions::Exception) : RObject(e_INTVEC), RDimensionInfo(), pVector(NULL), ulVecSize(0) {Initialise(ulLengthIn);};
		void Initialise(size_t ulLengthIn) throw(StdExtensions::Exception);
		// Import information from an R SEXP object
		virtual void ImportFromSEXP(SEXP pRObIn) throw(StdExtensions::Exception);
		// Export the vector as an R SEXP object
		virtual SEXP ExportAsSEXP() const throw(StdExtensions::Exception);
		// Function to return the length of the vector
		size_t Length() const throw() {return ulVecSize;};
		// Function to return elements from the vector
		StdExtensions::ExtInteger& GetElement(int iIter) const throw(StdExtensions::Exception);
		StdExtensions::ExtInteger& GetElement(const int *pIter) const throw(StdExtensions::Exception);
		StdExtensions::ExtInteger& operator [] (int iIter) const throw(StdExtensions::Exception) {return GetElement(iIter);};
		// Function to generate landscapes by hierarchical curdling
		void HierarchicalCurdling(size_t ulDimNumIn, size_t ulHierNum, const int *pExtents, int **pHierBoxes, const int *pHierProp, StdExtensions::Random *pRand) throw(StdExtensions::Exception);
		// Function to generate simple percolation maps
		void PercolationMap(size_t ulDimNumIn, size_t ulNumHabs, const int *pExtents, const int *pProps, StdExtensions::Random *pRand) throw(StdExtensions::Exception);
	};

	// -----------------
	//  R DOUBLE VECTOR
	// -----------------
	// Class to hold R double vector objects.
	class RDoubleVector : public RObject, public RDimensionInfo
	{
	private:
		StdExtensions::ExtDouble *pVector;			// Array of extended doubles
		size_t ulVecSize;							// Size of array
	public:
		// Null initialisation functions
		RDoubleVector() throw() : RObject(e_DBLVEC), RDimensionInfo(), pVector(NULL), ulVecSize(0) {};
		void Initialise() throw();
		virtual ~RDoubleVector() throw() {Initialise();};
		// Copy initialisation functions
		RDoubleVector(const RDoubleVector &cObIn) throw(StdExtensions::Exception) : RObject(e_DBLVEC), RDimensionInfo(), pVector(NULL), ulVecSize(0) {Initialise(cObIn);};
		void Initialise(const RDoubleVector &cObIn) throw(StdExtensions::Exception);
		RDoubleVector& operator = (const RDoubleVector &cObIn) throw(StdExtensions::Exception) {Initialise(cObIn); return *this;};
		// Parameterised initialisation functions
		RDoubleVector(size_t ulLengthIn) throw(StdExtensions::Exception) : RObject(e_DBLVEC), RDimensionInfo(), pVector(NULL), ulVecSize(0) {Initialise(ulLengthIn);};
		void Initialise(size_t ulLengthIn) throw(StdExtensions::Exception);
		// Import information from an R SEXP object
		virtual void ImportFromSEXP(SEXP pRObIn) throw(StdExtensions::Exception);
		// Export the vector as an R SEXP object
		virtual SEXP ExportAsSEXP() const throw(StdExtensions::Exception);
		// Function to return the length of the vector
		size_t Length() const throw() {return ulVecSize;};
		// Function to return elements from the vector
		StdExtensions::ExtDouble& GetElement(int iIter) const throw(StdExtensions::Exception);
		StdExtensions::ExtDouble& GetElement(const int *pIter) const throw(StdExtensions::Exception);
		StdExtensions::ExtDouble& operator [] (int iIter) const throw(StdExtensions::Exception) {return GetElement(iIter);};
		// Function that generates a spectral landscape with set Hurst exponents
		void SpectralSynthesis(size_t ulDimNumIn, const int *pExtents, const double *pHursts, StdExtensions::Random *pRand) throw(StdExtensions::Exception);
	};

	// ------------------
	//  R COMPLEX VECTOR
	// ------------------
	// Class to hold R complex vector objects
	class RComplexVector : public RObject, public RDimensionInfo
	{
	private:
		StdExtensions::ExtComplex *pVector;			// Array of extended complex values
		size_t ulVecSize;							// Size of array
	public:
		// Null initialisation functions
		RComplexVector() throw() : RObject(e_CPXVEC), RDimensionInfo(), pVector(NULL), ulVecSize(0) {};
		void Initialise() throw();
		virtual ~RComplexVector() throw() {Initialise();};
		// Copy initialisation functions
		RComplexVector(const RComplexVector &cObIn) throw(StdExtensions::Exception) : RObject(e_CPXVEC), RDimensionInfo(), pVector(NULL), ulVecSize(0) {Initialise(cObIn);};
		void Initialise(const RComplexVector &cObIn) throw(StdExtensions::Exception);
		RComplexVector& operator = (const RComplexVector &cObIn) throw(StdExtensions::Exception) {Initialise(cObIn); return *this;};
		// Parameterised initialisation functions
		RComplexVector(size_t ulLengthIn) throw(StdExtensions::Exception) : RObject(e_CPXVEC), RDimensionInfo(), pVector(NULL), ulVecSize(0) {Initialise(ulLengthIn);};
		void Initialise(size_t ulLengthIn) throw(StdExtensions::Exception);
		// Import information from an R SEXP object
		virtual void ImportFromSEXP(SEXP pRObIn) throw(StdExtensions::Exception);
		// Export the vector as an R SEXP object
		virtual SEXP ExportAsSEXP() const throw(StdExtensions::Exception);
		// Function to return the length of the vector
		size_t Length() const throw() {return ulVecSize;};
		// Function to return elements from the vector
		StdExtensions::ExtComplex& GetElement(int iIter) const throw(StdExtensions::Exception);
		StdExtensions::ExtComplex& GetElement(const int *pIter) const throw(StdExtensions::Exception);
		StdExtensions::ExtComplex& operator [] (int iIter) const throw(StdExtensions::Exception) {return GetElement(iIter);};
		// Function that calculates the (discrete) fast fourier transform of the target vector
		RComplexVector FFT(bool bInverse = false) const throw(StdExtensions::Exception);
		RComplexVector InvFFT() const throw(StdExtensions::Exception) {return FFT(true);};
		// Simulate the spectral density of fractional Brownian motion
		void FBMSpectralDensitySim(size_t ulDimNumIn, const int *pExtents, const double *pHursts, StdExtensions::Random *pRand) throw(StdExtensions::Exception);
	};

	// ------------------
	//  R BOOLEAN VECTOR
	// ------------------
	// Class to hold R boolean vector objects
	class RBoolVector : public RObject, public RDimensionInfo
	{
	private:
		StdExtensions::ExtBool *pVector;			// Array of extended boolean values
		size_t ulVecSize;							// Size of array
	public:
		// Null initialisation functions
		RBoolVector() throw() : RObject(e_BOOLVEC), RDimensionInfo(), pVector(NULL), ulVecSize(0) {};
		void Initialise() throw();
		virtual ~RBoolVector() throw() {Initialise();};
		// Copy initialisation functions
		RBoolVector(const RBoolVector &cObIn) throw(StdExtensions::Exception) : RObject(e_BOOLVEC), RDimensionInfo(), pVector(NULL), ulVecSize(0) {Initialise(cObIn);};
		void Initialise(const RBoolVector &cObIn) throw(StdExtensions::Exception);
		RBoolVector& operator = (const RBoolVector &cObIn) throw(StdExtensions::Exception) {Initialise(cObIn); return *this;};
		// Parameterised initialisation functions
		RBoolVector(size_t ulLengthIn) throw(StdExtensions::Exception) : RObject(e_BOOLVEC), RDimensionInfo(), pVector(NULL), ulVecSize(0) {Initialise(ulLengthIn);};
		void Initialise(size_t ulLengthIn) throw(StdExtensions::Exception);
		// Import information from an R SEXP object
		virtual void ImportFromSEXP(SEXP pRObIn) throw(StdExtensions::Exception);
		// Export the vector as an R SEXP object
		virtual SEXP ExportAsSEXP() const throw(StdExtensions::Exception);
		// Function to return the length of the vector
		size_t Length() const throw() {return ulVecSize;};
		// Function to return elements from the vector
		StdExtensions::ExtBool& GetElement(int iIter) const throw(StdExtensions::Exception);
		StdExtensions::ExtBool& GetElement(const int *pIter) const throw(StdExtensions::Exception);
		StdExtensions::ExtBool& operator [] (int iIter) const throw(StdExtensions::Exception) {return GetElement(iIter);};
	};

	// -----------------
	//  R STRING VECTOR
	// -----------------
	// Class to hold R sting vector objects
	class RStringVector : public RObject, public RDimensionInfo
	{
	private:
		StdExtensions::ExtString *pVector;			// Array of extended string values
		size_t ulVecSize;							// Size of array
	public:
		// Null initialisation functions
		RStringVector() throw() : RObject(e_STRVEC), RDimensionInfo(), pVector(NULL), ulVecSize(0) {};
		void Initialise() throw();
		virtual ~RStringVector() throw() {Initialise();};
		// Copy initialisation functions
		RStringVector(const RStringVector &cObIn) throw(StdExtensions::Exception) : RObject(e_STRVEC), RDimensionInfo(), pVector(NULL), ulVecSize(0) {Initialise(cObIn);};
		void Initialise(const RStringVector &cObIn) throw(StdExtensions::Exception);
		RStringVector& operator = (const RStringVector &cObIn) throw(StdExtensions::Exception) {Initialise(cObIn); return *this;};
		// Parameterised initialisation functions
		RStringVector(size_t ulLengthIn) throw(StdExtensions::Exception) : RObject(e_STRVEC), RDimensionInfo(), pVector(NULL), ulVecSize(0) {Initialise(ulLengthIn);};
		void Initialise(size_t ulLengthIn) throw(StdExtensions::Exception);
		// Import information from an R SEXP object
		virtual void ImportFromSEXP(SEXP pRObIn) throw(StdExtensions::Exception);
		// Export the vector as an R SEXP object
		virtual SEXP ExportAsSEXP() const throw(StdExtensions::Exception);
		// Function to return the length of the vector
		size_t Length() const throw() {return ulVecSize;};
		// Function to return elements from the vector
		StdExtensions::ExtString& GetElement(int iIter) const throw(StdExtensions::Exception);
		StdExtensions::ExtString& GetElement(const int *pIter) const throw(StdExtensions::Exception);
		StdExtensions::ExtString& operator [] (int iIter) const throw(StdExtensions::Exception) {return GetElement(iIter);};
	};

	// ---------------------
	//  R ENVIRONMENT CLASS
	// ---------------------
	// Class to store R environments
	class REnvironment : public RObject
	{
	private:
		SEXP pEnvironment;		// Pointer to environment closure
	public:
		// Null initialisation function
		REnvironment() throw() : RObject(e_ENV), pEnvironment(R_NilValue) {};
		void Initialise() throw();
		virtual ~REnvironment() throw() {Initialise();};
		// Copy initialisation function
		REnvironment(const REnvironment &cObIn) throw() : RObject(e_ENV), pEnvironment(R_NilValue) {Initialise(cObIn);};
		void Initialise(const REnvironment &cObIn) throw();
		REnvironment& operator = (const REnvironment &cObIn) throw() {Initialise(cObIn); return *this;};
		// Import information from an R SEXP object
		virtual void ImportFromSEXP(SEXP pRObIn) throw(StdExtensions::Exception);
		// Export the vector as an R SEXP object
		virtual SEXP ExportAsSEXP() const throw(StdExtensions::Exception);
	};

	// ------------------
	//  R LIST CONTAINER
	// ------------------
	// Class to hold R list objects.  This class holds both LISP style lists
	// and the newer vector style list.
	class RList : public RObject, public RDimensionInfo
	{
		friend class RListLinker;
		friend class RListIterator;
	public:
		// --------------
		//  LINKER CLASS
		// --------------
		// Class to link objects contained in the list
		class RListLinker
		{
			friend class RList;
			friend class RListIterator;
		private:
			RListLinker *pAfter;						// Pointer to the next linker in the list
			RListLinker *pBefore;						// Pointer to the previous linker in the list
			RObject *pObject;							// Pointer to the R object
			StdExtensions::ExtString cTitle;			// Title of the list object
			// Allocate a new copy of the input  object
			RObject *AllocateCopy(const RObject &cObject) throw(StdExtensions::Exception);
			// Parameterised initialisation
			RListLinker(RListLinker *pBeforeIn, RListLinker *pAfterIn, const RObject &cObject, const char *pTitle = NULL) throw(StdExtensions::Exception) : pAfter(NULL), pBefore(NULL), pObject(NULL), cTitle()
				{Initialise(pBeforeIn, pAfterIn, cObject, pTitle);};
			void Initialise(RListLinker *pBeforeIn, RListLinker *pAfterIn, const RObject &cObject, const char *pTitle = NULL) throw(StdExtensions::Exception);
		public:
			// Null initialisation
			RListLinker() throw() : pAfter(NULL), pBefore(NULL), pObject(NULL), cTitle() {};
			void Initialise() throw();
			// Retrieve the title of the linker element
			StdExtensions::ExtString GetTitle() const throw(StdExtensions::Exception) {return cTitle;};
			char *GetTitle(char *pStringOut, size_t ulSize) const throw() {return cTitle.RetrieveCString(pStringOut, ulSize);};
			// Set the title of the linker element
			void SetTitle(const StdExtensions::ExtString &cTitleIn) throw(StdExtensions::Exception) {cTitle.Initialise(cTitleIn);};
			void SetTitle(const char *pTitleIn, size_t ulSize) throw(StdExtensions::Exception) {cTitle.Initialise(pTitleIn, ulSize);};
			void SetTitle(const char *pTitleIn) throw(StdExtensions::Exception) {cTitle.Initialise(pTitleIn);};
			// Retrieve the object pointed at by the list linker
			RObject *GetObject() const throw() {return const_cast<RObject *>(pObject);};
			// Get pointers to the adjacent list linker objects
			RListLinker *GetNext() const throw() {return const_cast<RListLinker *>(pAfter);};
			RListLinker *GetPrevious() const throw() {return const_cast<RListLinker *>(pBefore);};
			~RListLinker() throw() {Initialise();};
		};
		// ----------------
		//  ITERATOR CLASS
		// ----------------
		// Class for list iteration
		class RListIterator
		{
			friend class RList;
			friend class RListLinker;
		private:
			RListLinker *pCurElement;	// Pointer to the current
			RList *pParent;				// Pointer to the parent list
		public:
			// Null initialisation
			RListIterator() throw() : pCurElement(NULL), pParent(NULL) {};
			void Initialise() throw();
			~RListIterator() throw() {Initialise();};
			// Copy initialisation
			RListIterator(const RListIterator &cObIn) throw(StdExtensions::Exception) : pCurElement(NULL), pParent(NULL) {Initialise(cObIn);};
			void Initialise(const RListIterator &cObIn) throw(StdExtensions::Exception);
			RListIterator& operator = (const RListIterator &cObIn) throw(StdExtensions::Exception) {Initialise(cObIn); return *this;};
			// Retrieve object
			RObject& GetObject() const throw(StdExtensions::Exception);
			// Increment / decrement the iterators
			void Increment() throw();
			void Increment(int iNum) throw();
			void Decrement() throw();
			void Decrement(int iNum) throw() {Increment(-iNum);};
			// Overloaded arithmetic operators
			//		overloaded pointer actualisation operator
			RObject& operator * () const throw(StdExtensions::Exception) {return GetObject();};
			//		increment / decrement operators
			RListIterator& operator ++ () throw() {Increment(); return *this;};
			RListIterator operator ++ (int) throw(StdExtensions::Exception) {RListIterator cTemp(*this); Increment(); return cTemp;};
			RListIterator& operator -- () throw() {Decrement(); return *this;};
			RListIterator operator -- (int) throw(StdExtensions::Exception) {RListIterator cTemp(*this); Decrement(); return cTemp;};
			//		arithmetic assignment operators
			RListIterator& operator += (int iIter) throw() {Increment(iIter); return *this;};
			RListIterator& operator -= (int iIter) throw() {Decrement(iIter); return *this;};
			//		arithmetic operators
			RListIterator operator + (int iIter) const throw(StdExtensions::Exception) {RListIterator cTemp(*this); cTemp.Increment(iIter); return cTemp;};
			RListIterator operator - (int iIter) const throw(StdExtensions::Exception) {RListIterator cTemp(*this); cTemp.Decrement(iIter); return cTemp;};
			// Check to ensure that the iterator is valid
			bool IsValid() const throw() {return pCurElement != NULL;};
			// Get the title of the current object
			char *GetTitle(char *pStringOut, size_t ulSize) const throw(StdExtensions::Exception);
		};
	private:
		mutable RListIterator **pIterCache;		// Cache of iterators
		mutable size_t ulIterCacheSize;			// Size of the iterator cache
		RListLinker *pStart;					// Start of the list of objects
		RListLinker *pFinish;					// End of the list of objects
		// Add an iterator to the cache
		void AddIteratorToCache(const RListIterator *pIterIn) const throw(StdExtensions::Exception);
	public:
		// Null initialisation functions
		RList() throw() : RObject(e_LIST), RDimensionInfo(), pIterCache(NULL), ulIterCacheSize(0),
			pStart(NULL), pFinish(NULL) {};
		void Initialise() throw();
		~RList() throw() {Initialise();};
		// Copy initialisation function
		RList(const RList &cObIn) throw(StdExtensions::Exception) : RObject(e_LIST), pIterCache(NULL), ulIterCacheSize(0),
			pStart(NULL), pFinish(NULL) {Initialise(cObIn);};
		void Initialise(const RList &cObIn) throw(StdExtensions::Exception);
		RList& operator = (const RList &cObIn) throw(StdExtensions::Exception) {Initialise(cObIn); return *this;};
		// Add R object to the list
		void AddElementToFront(const RObject &cObIn, const StdExtensions::ExtString &cTitle) throw(StdExtensions::Exception);
		void AddElementToBack(const RObject &cObIn, const StdExtensions::ExtString &cTitle) throw(StdExtensions::Exception);
		void AddElementToFront(const RObject &cObIn, const char *pTitle) throw(StdExtensions::Exception)
			{StdExtensions::ExtString cTitle; cTitle.Initialise(pTitle); AddElementToFront(cObIn, cTitle);};
		void AddElementToBack(const RObject &cObIn, const char *pTitle) throw(StdExtensions::Exception)
			{StdExtensions::ExtString cTitle; cTitle.Initialise(pTitle); AddElementToBack(cObIn, cTitle);};
		void AddElementToFront(const RObject &cObIn) throw(StdExtensions::Exception)
			{StdExtensions::ExtString cTitle; cTitle.Initialise(); AddElementToFront(cObIn, cTitle);};
		void AddElementToBack(const RObject &cObIn) throw(StdExtensions::Exception)
			{StdExtensions::ExtString cTitle; cTitle.Initialise(); AddElementToBack(cObIn, cTitle);};
		// Import information from an R SEXP object
		virtual void ImportFromSEXP(SEXP pRObIn) throw(StdExtensions::Exception);
		// Fill from parameters given to the .External function
		void FillFromExternalParameters(SEXP pRObIn) throw(StdExtensions::Exception);
		// Export the list as an R SEXP object
		virtual SEXP ExportAsSEXP() const throw(StdExtensions::Exception);
		// Function to return the length of the list
		size_t Length() const throw();
		// Create iterators at the start or the end of the list
		RListIterator CreateNewIteratorFront() const throw(StdExtensions::Exception);
		RListIterator CreateNewIteratorBack() const throw(StdExtensions::Exception);
		// Access objects in the list according to an index variable
		RObject& GetElement(int iIter) const throw(StdExtensions::Exception);
		RObject& GetElement(const int *pIter) const throw(StdExtensions::Exception);
		RObject& GetElement(const char *pEl) const throw(StdExtensions::Exception) {StdExtensions::ExtString cTitle; cTitle.Initialise(pEl); return GetElement(cTitle);};
		RObject& GetElement(const StdExtensions::ExtString &cTitle) const throw(StdExtensions::Exception);
		RObject& operator [] (int iIter) const throw(StdExtensions::Exception) {return GetElement(iIter);};
	};

	// Overloaded arithmetic operators for iterator incrementing
	inline RList::RListIterator operator + (int iIter, const RList::RListIterator &cObIn) throw(StdExtensions::Exception) {RList::RListIterator cTemp(cObIn); cTemp.Increment(iIter); return cTemp;}
	inline RList::RListIterator operator - (int iIter, const RList::RListIterator &cObIn) throw(StdExtensions::Exception) {RList::RListIterator cTemp(cObIn); cTemp.Decrement(iIter); return cTemp;}

	// ------------------
	//  R FUNCTION CLASS
	// ------------------
	// Class to store and use R functions
	class RFunction : public RObject
	{
	private:
		SEXP pFunction;			// Pointer to function enclosure
	public:
		// Null initialisation function
		RFunction() throw() : RObject(e_FUNC), pFunction(R_NilValue) {};
		void Initialise() throw();
		virtual ~RFunction() throw() {Initialise();};
		// Copy initialisation function
		RFunction(const RFunction &cObIn) throw() : RObject(e_FUNC), pFunction(R_NilValue) {Initialise(cObIn);};
		void Initialise(const RFunction &cObIn) throw();
		RFunction& operator = (const RFunction &cObIn) throw() {Initialise(cObIn); return *this;};
		// Call the function using the list as parameters
		SEXP CallFunction(const RList &cObIn) throw(StdExtensions::Exception);
		// Import information from an R SEXP object
		virtual void ImportFromSEXP(SEXP pRObIn) throw();
		// Export the vector as an R SEXP object
		virtual SEXP ExportAsSEXP() const throw();
	};

	// Set of functions to call R functions
	inline SEXP CallRFunction(RFunction &cFunc, const RList &cParams) throw(StdExtensions::Exception) {return cFunc.CallFunction(cParams);}
	SEXP CallRFunction(const char *pFunc, const RList &cParams) throw(StdExtensions::Exception);
}

//Return warnings to default upon compilation with Visual C++ compiler
#ifdef _MSC_VER
	#pragma warning( default : 4290 )
#endif

#endif
