#include "stdafx.h"
#include "Random.h"

//If the file is being compiled with a Visual C++ compiler then set it to be included once in the program only
//and to temporarily disable exception specification warnings generated as a result of non-standard behaviour
#ifdef _MSC_VER
	#pragma once
	#pragma warning( disable : 4290 )
#endif

namespace StdExtensions
{
	// Null initialisation function
	void Random::Initialise() throw()
	{	// Reset normal settings
		bNormEx = false;
		dNormEx = 0.0;
	}

	// Generate a uniform random number between dA and dB
	double Random::Uniform(double dA, double dB) throw()
	{
		double dMin, dMax, dOut;
		dMin = dMax = dOut = 0.0;
		if(dA == dB)
		{	// If inputs are equal then return the value they are equal to
			dOut = dA;
		}
		else
		{	// Calculate minimum and maximum values
			dMin = dA < dB ? dA : dB;
			dMax = dA > dB ? dA : dB;
			// Rescale standard uniform function
			dOut = dMin + (dMax - dMin) * UniformGen();
		}
		return dOut;
	}

	// Generate a discrete uniform random number between dA and dB
	int Random::DiscreteUniform(int iA, int iB) throw()
	{
		double dMin, dMax;
		int iOut;
		dMin = dMax = 0.0;
		iOut = 0;
		if(iA == iB)
		{	// If inputs are equal then return the value they are equal to
			iOut = iA;
		}
		else
		{	// Calculate minimum and maximum values
			dMin = double(iA < iB ? iA : iB);
			dMax = double(iA > iB ? iA : iB) + 1.0;
			// Rescale standard uniform function
			iOut = int(Uniform(dMin, dMax));
		}
		return iOut;
	}

	// Generate a normal random number with mean dMean and standard deviation dSD
	double Random::Normal(double dMean, double dSD) throw(StdExtensions::Exception)
	{
		double dUnif1, dUnif2, dDist, dOut;
		dUnif1 = dUnif2 = dDist = dOut = 0.0;
		if(dSD <= 0.0)
		{	// Raise exception if invalid value for the standard deviation is given
			StdExtensions::RaiseException("invalid value for the standard deviation", __LINE__, "StdExtensions::Random::Normal(double, double)", __FILE__, StdExtensions::e_DomainError);
		}
		if(bNormEx)
		{	// A normal variate has already been stored - use that
			bNormEx = false;
			dOut = dNormEx;
			dNormEx = 0.0;
		}
		else
		{	// Other wise generate two random variables using the polar version of the
			// Box-Muller transform
			do
			{
				dUnif1 = 2.0 * UniformGen() - 1.0;
				dUnif2 = 2.0 * UniformGen() - 1.0;
				dDist = StdExtensions::Sqrd(dUnif1) + StdExtensions::Sqrd(dUnif2);
			} while(dDist >= 1.0);
			dDist = sqrt((-2.0 * log(dDist)) / dDist);
			// Store one of the random variables for use next time
			dNormEx = dUnif1 * dDist;
			bNormEx = true;
			dOut = dUnif2 * dDist;
		}
		return dMean + dSD * dOut;
	}
}

//Return warnings to default upon compilation with Visual C++ compiler
#ifdef _MSC_VER
	#pragma warning( default : 4290 )
#endif
